@@ -40,18 +40,24 @@ typedef struct
 	Proxy *proxy;
 	/* login widget start */
 	GtkWidget* fixed;
+	GtkWidget* portrait;
 	GtkWidget* username;
 	GtkWidget* password;
 	GtkWidget* loginbutton;
-	GtkWidget* label;
 	GtkWidget* userlabel;
 	GtkWidget* passlabel;
+	GtkWidget* statusLabel;
 	GtkWidget* statecombo;
 	GtkWidget* remember;
 	GtkWidget* proxyBtn;
 	GtkWidget* proxyLabel;
 	GtkListStore* store;
 	GtkTreeModel* model;
+	/* logining panel */
+	GtkWidget* fixed1;
+	GtkWidget* image;
+	GtkWidget* label;
+	GtkWidget* errlabel;
 } FxLogin;
 
 typedef struct
@@ -95,7 +101,8 @@ typedef struct
 	FxList* tlist;				 /*  time out args list							 */
 	FxList* shlist;				 /*	 share content window list		   			 */
 	FxList* pglist;
-#ifdef HAVE_LIBNOTIFY
+	FetionHistory* history;
+#ifdef USE_LIBNOTIFY
 	NotifyNotification *notify;
 #endif
 	int iconConnectId;			 /*  status icon connect id						 */
@@ -228,11 +235,11 @@ typedef struct
 typedef struct
 {
 	FxMain* fxmain;
-	FetionHistory* fhistory;
 	Conversation* conv;
 	ChatDialogFocusedType hasFocus;
 	gboolean sendtophone;
 	StateType state;
+	int unreadMsgCount;
 	/* main widget begin */
 	GtkWidget* dialog;
 	GtkWidget* headbox;
@@ -391,6 +398,8 @@ typedef struct
 	GtkWidget *etBtn;
 	GtkWidget *alertBtn;
 	GtkWidget *iconBtn;
+	GtkWidget *autoAwayBtn;
+	GtkWidget *onlineNotifyBtn;
 	/* personal setting */
 	GtkWidget* image ;
 	GtkWidget* sid_label ;
