@@ -24,14 +24,6 @@
 
 /* inline function to trace program track */
 
-#ifdef FETION_DEBUG_TRACE
-#	define DEBUG_FOOTPRINT()	\
-		printf("| TRACK |function '%s()' in file: <%s:%d>\n" \
-		, __FUNCTION__ , __FILE__ , __LINE__)
-#else
-#	define DEBUG_FOOTPRINT()
-#endif
-
 #ifdef UNUSED
 #elif defined(__GNUC__)
 # 	define UNUSED(x) UNUSED_ ## x __attribute__((unused))
@@ -40,7 +32,7 @@
 #endif
 
 #define FETION_NAME "OpenFetion"
-#define FETION_VERSION "1.9"
+#define FETION_VERSION "2.0.1"
 #define PROTO_VERSION "4.0.2510"
 #define NAVIGATION_URI "nav.fetion.com.cn"
 #define PGGROUP_SPACE_URI "http://group.feixin.10086.cn/space/Index/"
@@ -92,7 +84,8 @@ typedef enum
 {
 	IMAGE_NOT_INITIALIZED = -1 ,		/* portrait has not been initialized */
 	IMAGE_NOT_CHANGED ,					/* portrait does not change 		 */
-	IMAGE_CHANGED						/* portrait has been changed 		 */
+	IMAGE_CHANGED ,						/* portrait has been changed 		 */
+	IMAGE_ALLREADY_SET
 } ImageChangedType;
 
 /**
@@ -175,9 +168,9 @@ typedef struct contact
 	char userId[16];					/* userid used since v4 protocal      				*/
 	char sId[16];						/* fetion no					      				*/
 	char sipuri[48];					/* sipuri like 'sip:100@fetion.com.cn'				*/
-	char localname[32];					/* name set by yourself				  				*/
-	char nickname[32];					/* user`s nickname					    			*/
-	char impression[1024];				/* user`s mood phrase				    			*/
+	char localname[256];					/* name set by yourself				  				*/
+	char nickname[256];					/* user`s nickname					    			*/
+	char impression[2048];				/* user`s mood phrase				    			*/
 	char mobileno[12];					/* mobile phone number				    			*/
 	char devicetype[10];				/* user`s client type , like PC and J2ME,etc		*/
 	char portraitCrc[12];				/* a number generated by crc algorithm 				*/
@@ -195,6 +188,8 @@ typedef struct contact
 	int groupid;						/* buddylist id										*/
 	int gender;							/* gender 1 for male 2 for female,0 for private		*/
 	int imageChanged;					/* whether user`s portrait has changed				*/
+	int dirty;                          /* whether the contact just read from the server is 
+										   newer than that int the local disk */
 	struct contact* next;
 	struct contact* pre;
 } Contact;
@@ -206,6 +201,7 @@ typedef struct group
 {
 	char groupname[32];					/* current buddy list name  */
 	int groupid;						/* current buddy list Id	*/
+	int dirty;
 	struct group *next;
 	struct group *pre;
 } Group;
@@ -271,6 +267,8 @@ struct userlist
 {
 	char no[24];						/* fetion no or mobile no  		*/
 	char password[48];					/* password 			   		*/
+	char userid[48];
+	char sid[48];
 	int laststate;						/* last state when logining		*/
 	int islastuser;						/* is the last logined user		*/
 	struct userlist *pre;
@@ -320,6 +318,14 @@ typedef struct
 	int closeMode;						/* close button clicked to close window or iconize it	  */
 	int canIconify;
 	int allHighlight;
+	int autoAway;
+	int autoAwayTimeout;
+	int onlineNotify;
+
+	char configServersVersion[16];		/* the version of some related servers such as sipc server	*/
+	char configParametersVersion[16];
+	char configHintsVersion[16];		/* the version of hints										*/
+
 	struct userlist* ul;				/* user list stored in local data file					  */
 	Proxy *proxy;						/* structure stores the global proxy information 		  */
 } Config;
@@ -342,9 +348,6 @@ typedef struct
 	char contactVersion[16];			/* the version of contact information						*/
 	char customConfigVersion[16];		/* the version of custom config string,unused now			*/
 
-	char configServersVersion[16];		/* the version of some related servers such as sipc server	*/
-	char configHintsVersion[16];		/* the version of hints										*/
-
 	char nickname[48];					/* nickname of yourself										*/
 	char impression[256];				/* mood phrase of yourself									*/
 	char portraitCrc[16];				/* a number generated by crc algorithm						*/
@@ -368,6 +371,8 @@ typedef struct
 	int carrierStatus;
 	int boundToMobile;					/* whether this number is bound to a mobile number  */
 	long loginTimes;
+	int contactCount;
+	int groupCount;
 	char* ssic;						    /* cookie string read from reply message after ssi login 	*/
 	char* customConfig;					/* custom config string used to set personal information	*/
 	Verification* verification;			/* a struct used to generate picture code					*/	 
@@ -409,15 +414,15 @@ typedef struct
 {
 	char name[48];						 /* name of message sender	   */
 	char userid[16];					 /* userid of message sender   */
-	struct tm sendtime;					 /* message sent time		   */
-	char message[550];					 /* message content			   */
+	char sendtime[32];					 /* message sent time		   */
+	char message[4096];					 /* message content			   */
 	int issend;							 /* message is sent of received*/	
 } History;
 
 typedef struct
 {
 	User* user;
-	FILE* file;
+	sqlite3 *db;
 } FetionHistory;
 
 #define FILE_RECEIVE 1
