@@ -22,28 +22,21 @@
 /*private */
 
 static char* generate_subscribe_body(const char* version);
-
 static char* generate_contact_info_body(const char* userid);
-
 static char* generate_contact_info_by_no_body(const char* no , NumberType nt);
-
 static char* generate_set_mobileno_perssion(const char* userid , int show);
-
 static char* generate_set_displayname_body(const char* userid , const char* name);
-
 static char* generate_move_to_group_body(const char* userid , int buddylist);
-
 static char* generate_delete_buddy_body(const char* userid);
-
 static char* generate_add_buddy_body(const char* no , NumberType notype
 								, int buddylist , const char* localname
 								, const char* desc , int phraseid);
-
 static char* generate_handle_contact_request_body(const char* sipuri
 								, const char* userid , const char* localname
 								, int buddylist , int result );
 static Contact* parse_handle_contact_request_response(const char* sipmsg);
 static Contact* parse_add_buddy_response(const char* sipmsg , int* statuscode);
+static int has_special_word(const char *in);
 
 Contact* fetion_contact_new()
 {
@@ -66,7 +59,6 @@ Contact* fetion_contact_list_find_by_userid(Contact* contactlist , const char* u
 {
 	Contact* cl_cur;
 	foreach_contactlist(contactlist , cl_cur){
-		//printf("%s , %s\n" , cl_cur->userId , userid);
 		if(strcmp(cl_cur->userId , userid) == 0)
 			return cl_cur;
 	}
@@ -101,6 +93,12 @@ void fetion_contact_list_remove_by_userid(Contact* contactlist , const char* use
 		}
 	}
 }
+
+void fetion_contact_list_remove(Contact *contact)
+{
+	contact->next->pre = contact->pre;
+	contact->pre->next = contact->next;
+}
 void fetion_contact_list_free(Contact* contact)
 {
 	Contact *cl_cur , *del_cur;
@@ -135,11 +133,11 @@ Contact* fetion_contact_get_contact_info(User* user , const char* userid)
 {
 	FetionSip* sip = user->sip;
 	char *res , *body , *pos;
+	char *cur;
 	Contact* contact;
 	xmlChar* cs;
 	xmlDocPtr doc;
 	xmlNodePtr node;
-	int n;
 	contact = fetion_contact_list_find_by_userid(user->contactList , userid);
 	body = generate_contact_info_body(userid);
 	fetion_sip_set_type(sip , SIP_SERVICE);
@@ -149,25 +147,25 @@ Contact* fetion_contact_get_contact_info(User* user , const char* userid)
 	tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res);
 	res = fetion_sip_get_response(sip);
-	if(res == NULL){
+	if(res == NULL)
 		return NULL;
-	}
+
 	pos = strstr(res , "\r\n\r\n") + 4;
 	doc = xmlParseMemory(pos , strlen(pos));
+	if(!doc)
+		return NULL;
 	node = xmlDocGetRootElement(doc);
 	node = node->xmlChildrenNode;
 	if(xmlHasProp(node , BAD_CAST "carrier-region")){
 		cs = xmlGetProp(node , BAD_CAST "carrier-region");
 		pos = (char*)cs;
-		n = strlen(pos) - strlen(strstr(pos , "."));
-		strncpy(contact->country , pos , n);
-		pos = strstr(pos , ".") + 1;
-		n = strlen(pos) - strlen(strstr(pos , "."));
-		strncpy(contact->province , pos , n);
-		pos = strstr(pos , ".") + 1;
-		n = strlen(pos) - strlen(strstr(pos , "."));
-		strncpy(contact->city , pos , n);
-		contact->city[n] = 0;
+
+		for(cur = contact->country;*pos && *pos != '.';*cur ++ = *pos ++);
+		*cur = '\0'; pos ++;
+		for(cur = contact->province;*pos && *pos != '.';*cur ++ = *pos ++);
+		*cur = '\0'; pos ++;
+		for(cur = contact->city;*pos && *pos != '.';*cur ++ = *pos ++);
+		*cur = '\0';
 		xmlFree(cs);
 		free(res);
 	}
@@ -211,19 +209,19 @@ Contact* fetion_contact_get_contact_info_by_no(User* user , const char* no , Num
 	body = generate_contact_info_by_no_body(no , nt);
 	res = fetion_sip_to_string(sip , body);
 	free(body);
-	tcp_connection_send(sip->tcp , res , strlen(res));
+	ret = tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res); res = NULL; 
+	if(ret < 0)
+		return NULL;
+
 	res = fetion_sip_get_response(sip);
 	ret = fetion_sip_get_code(res);
-	if(ret == 200)
-	{
+	if(ret == 200){
 		contact = parse_contact_info_by_no_response(res);
 		free(res);
 		debug_info("Get user information by mobile number success");
 		return contact;
-	}
-	else
-	{
+	}else{
 		free(res);
 		debug_error("Get user information by mobile number failed , errno :" , ret);
 		return NULL;
@@ -241,19 +239,19 @@ int fetion_contact_set_mobileno_permission(User* user , const char* userid , int
 	body = generate_set_mobileno_perssion(userid , show);
 	res = fetion_sip_to_string(sip , body);
 	free(body);
-	tcp_connection_send(sip->tcp , res , strlen(res));
+	ret = tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res) ; 
+	if(ret < 0)
+		return -1;
+
 	res = fetion_sip_get_response(sip);
 	ret = fetion_sip_get_code(res);
-	if(ret == 200)
-	{
+	if(ret == 200){
 		parse_set_mobileno_permission_response(user , res);
 		free(res);
 		debug_info("Get user information by mobile number success");
 		return 1;
-	}
-	else
-	{
+	}else{
 		free(res);
 		debug_error("Get user information by mobile number failed , errno :" , ret);
 		return -1;
@@ -272,18 +270,18 @@ int fetion_contact_set_displayname(User* user , const char* userid , const char*
 	body = generate_set_displayname_body(userid , name);
 	res = fetion_sip_to_string(sip , body);
 	free(body);
-	tcp_connection_send(sip->tcp , res , strlen(res));
+	ret = tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res);
+	if(ret < 0)
+		return -1;
 	res = fetion_sip_get_response(sip);
 	ret = fetion_sip_get_code(res);
 	free(res);
-	if(ret == 200)
-	{
+
+	if(ret == 200){
 		debug_info("Set buddy(%s)`s localname to %s success" , userid , name);
 		return 1;
-	}
-	else
-	{
+	}else{
 		debug_info("Set buddy(%s)`s localname to %s failed" , userid , name);
 		return -1;
 	}
@@ -300,18 +298,20 @@ int fetion_contact_move_to_group(User* user , const char* userid , int buddylist
 	body = generate_move_to_group_body(userid , buddylist);
 	res = fetion_sip_to_string(sip , body);
 	free(body);
-	tcp_connection_send(sip->tcp , res , strlen(res));
+	ret = tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res);
+
+	if(ret < 0)
+		return -1;
+
 	res = fetion_sip_get_response(sip);
 	ret = fetion_sip_get_code(res);
 	free(res);
-	if(ret == 200)
-	{
+
+	if(ret == 200){
 		debug_info("Move buddy(%s) to group %d success" , userid , buddylist);
 		return 1;
-	}
-	else
-	{
+	}else{
 		debug_info("Move buddy(%s) to group %d failed" , userid , buddylist);
 		return -1;
 	}
@@ -328,19 +328,26 @@ int fetion_contact_delete_buddy(User* user , const char* userid)
 	body = generate_delete_buddy_body(userid);
 	res = fetion_sip_to_string(sip , body);
 	free(body);
-	tcp_connection_send(sip->tcp , res , strlen(res));
+#if 0
+	if(fetion_contact_del_localbuddy(user, userid) == -1)
+		return -1;
+#endif
+
+	ret = tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res);
+
+	if(ret < 0)
+		return -1;
+
 	res = fetion_sip_get_response(sip);
 	ret = fetion_sip_get_code(res);
 	free(res);
-	if(ret == 200)
-	{
+
+	if(ret == 200){
 		fetion_contact_list_remove_by_userid(user->contactList , userid);
 		debug_info("Delete buddy(%s) success" , userid);
 		return 1;
-	}
-	else
-	{
+	}else{
 		debug_info("Delete buddy(%s) failed" , userid);
 		return -1;
 	}
@@ -472,7 +479,6 @@ char* generate_contact_info_by_no_body(const char* no , NumberType nt)
 	xmlNodePtr node;
 	char uri[32];
 	char body[] = "<args></args>";
-	bzero(uri , sizeof(uri));
 	if(nt == MOBILE_NO)
 		sprintf(uri , "tel:%s" , no);
 	else
@@ -492,7 +498,6 @@ char* generate_set_mobileno_perssion(const char* userid , int show)
 	xmlNodePtr node;
 	char permission[32];
 	char body[] = "<args></args>";
-	bzero(permission , sizeof(permission));
 	sprintf(permission , "identity=%d" , show);
 	doc = xmlParseMemory(body , strlen(body));
 	node = xmlDocGetRootElement(doc);
@@ -521,9 +526,7 @@ char* generate_handle_contact_request_body(const char* sipuri
 	node = xmlNewChild(node , NULL , BAD_CAST "buddy" , NULL);
 	xmlNewProp(node , BAD_CAST "user-id" , BAD_CAST userid);
 	xmlNewProp(node , BAD_CAST "uri" , BAD_CAST sipuri);
-	bzero(result_s , sizeof(result_s));
 	sprintf(result_s , "%d" , result);
-	bzero(buddylist_s , sizeof(buddylist_s));
 	sprintf(buddylist_s , "%d" , buddylist);
 	xmlNewProp(node , BAD_CAST "result" , BAD_CAST result_s);
 	xmlNewProp(node , BAD_CAST "buddy-lists" , BAD_CAST buddylist_s);
@@ -562,7 +565,6 @@ char* generate_move_to_group_body(const char* userid , int buddylist)
 	node = xmlNewChild(node , NULL , BAD_CAST "contacts" , NULL);
 	node = xmlNewChild(node , NULL , BAD_CAST "contact" , NULL);
 	xmlNewProp(node , BAD_CAST "user-id" , BAD_CAST userid);
-	bzero(bl , sizeof(bl));
 	sprintf(bl , "%d" , buddylist);
 	xmlNewProp(node , BAD_CAST "buddy-lists" , BAD_CAST bl);
 	xmlDocDumpMemory(doc , &res , NULL);
@@ -601,13 +603,12 @@ char* generate_add_buddy_body(const char* no
 	node = xmlNewChild(node , NULL , BAD_CAST "contacts" , NULL);
 	node = xmlNewChild(node , NULL , BAD_CAST "buddies" , NULL);
 	node = xmlNewChild(node , NULL , BAD_CAST "buddy" , NULL);
-	bzero(uri , sizeof(uri));
-	bzero(phrase , sizeof(phrase));
-	bzero(groupid , sizeof(groupid));
+
 	if(notype == FETION_NO)
 		sprintf(uri , "sip:%s" , no);
 	else
 		sprintf(uri , "tel:%s" , no);
+
 	sprintf(phrase , "%d" , phraseid);
 	sprintf(groupid , "%d" , buddylist);
 	xmlNewProp(node , BAD_CAST "uri" , BAD_CAST uri);
@@ -761,8 +762,6 @@ static Contact* parse_add_buddy_response(const char* sipmsg , int* statuscode)
 	node = xmlDocGetRootElement(doc);
 	node = xml_goto_node(node , "buddy");
 
-	printf("%s\n" , sipmsg);
-
 	if(node == NULL)
 	{
 		*statuscode = 400;
@@ -827,8 +826,6 @@ Contact* parse_handle_contact_request_response(const char* sipmsg)
 	xmlNodePtr node;
 	contact = fetion_contact_new();
 
-	printf("%s\n" , sipmsg);
-
 	pos = strstr(sipmsg , "\r\n\r\n") + 4;
 	doc = xmlParseMemory(pos , strlen(pos));
 	node = xmlDocGetRootElement(doc);
@@ -911,3 +908,314 @@ void parse_add_buddy_verification(User* user , const char* str)
 	xmlFree(res);
 	user->verification = ver;
 }
+
+void fetion_contact_load(User *user, int *gcount, int *bcount)
+{
+	char path[256];
+	char sql[4096];
+	sqlite3 *db;
+	char *errMsg = NULL;
+	char **sqlres;
+	int ncols, nrows, i, j, start;
+	Contact *pos;
+	Group *gpos;
+	Config *config = user->config;
+
+	debug_info("Load contact list");
+
+	*gcount = 0;
+	*bcount = 0;
+
+	sprintf(path , "%s/data.db" , config->userPath);
+	if(sqlite3_open(path, &db)){
+		debug_error("failed to load contact list");
+		return;
+	}
+
+	sprintf(sql, "select * from groups order by groupid;");
+	if(sqlite3_get_table(db, sql, &sqlres,
+						&nrows, &ncols, &errMsg)){
+		sqlite3_close(db);
+		return;
+	}
+
+	*gcount = nrows;
+
+	for(i = 0; i < nrows; i++){
+		gpos = fetion_group_new();
+		for(j = 0; j < ncols; j++){
+			start = ncols + i * ncols;
+			gpos->groupid = atoi(sqlres[start]);
+			strcpy(gpos->groupname, sqlres[start+1]);
+		}
+		fetion_group_list_append(user->groupList, gpos);
+	}
+	sqlite3_free_table(sqlres);
+
+	sprintf(sql, "select * from contacts;");
+	if(sqlite3_get_table(db, sql, &sqlres,
+						&nrows, &ncols, &errMsg)){
+		sqlite3_close(db);
+		return;
+	}
+
+	*bcount = nrows;
+	
+	for(i = 0; i < nrows; i++){
+		pos = fetion_contact_new();
+		for(j = 0; j < ncols; j++){
+			start = ncols + i * ncols;
+			strcpy(pos->userId, 	sqlres[start]);
+			strcpy(pos->sId, 		sqlres[start+1]);
+			strcpy(pos->sipuri, 	sqlres[start+2]);
+			strcpy(pos->localname,  sqlres[start+3]);
+			strcpy(pos->nickname, 	sqlres[start+4]);
+			strcpy(pos->impression, sqlres[start+5]);
+			strcpy(pos->mobileno, 	sqlres[start+6]);
+			strcpy(pos->devicetype, sqlres[start+7]);
+			strcpy(pos->portraitCrc,sqlres[start+8]);
+			strcpy(pos->birthday, 	sqlres[start+9]);
+			strcpy(pos->country, 	sqlres[start+10]);
+			strcpy(pos->province, 	sqlres[start+11]);
+			strcpy(pos->city, 		sqlres[start+12]);
+			pos->identity = 		atoi(sqlres[start+13]);
+			pos->scoreLevel = 		atoi(sqlres[start+14]);
+			pos->serviceStatus = 	atoi(sqlres[start+15]);
+			pos->carrierStatus = 	atoi(sqlres[start+16]);
+			pos->relationStatus = 	atoi(sqlres[start+17]);
+			strcpy(pos->carrier,	sqlres[start+18]);
+			pos->groupid = 			atoi(sqlres[start+19]);
+			pos->gender = 			atoi(sqlres[start+20]);
+		}
+		fetion_contact_list_append(user->contactList, pos);
+	}
+	sqlite3_close(db);
+	sqlite3_free_table(sqlres);
+}
+
+void fetion_contact_save(User *user)
+{
+	char path[256];
+	char sql[4096];
+	sqlite3 *db;
+	char *errMsg = NULL;
+	Contact *pos;
+	Group *gpos;
+	Config *config = user->config;
+
+	debug_info("Save contact list");
+
+	sprintf(path , "%s/data.db" , config->userPath);
+	if(sqlite3_open(path, &db)){
+		debug_error("failed to save user list");
+		return;
+	}
+	/* begin transaction */
+	if(sqlite3_exec(db, "BEGIN TRANSACTION;", 0,0, &errMsg)){
+		debug_error("begin transaction :%s", errMsg);
+		sqlite3_close(db);
+		return;
+	}
+
+	sprintf(sql, "delete from groups");
+	if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+		sprintf(sql, "create table groups (groupid,groupname)");
+		if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+			debug_error("create table groups:%s",
+							sqlite3_errmsg(db));
+			sqlite3_close(db);
+			return;
+		}
+	}
+	foreach_grouplist(user->groupList, gpos){
+		snprintf(sql, sizeof(sql)-1, "insert into groups "
+				"values (%d,'%s');", gpos->groupid,
+				gpos->groupname);
+		if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+			debug_error("insert group info:%s",
+							sqlite3_errmsg(db));
+			continue;
+		}
+	}
+	
+	sprintf(sql, "delete from contacts;");
+	if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+		sprintf(sql, "create table contacts (userId,"
+						"sId,sipuri,localname,nickname,"
+						"impression,mobileno,devicetype,"
+						"portraitCrc,birthday,country,"
+						"province,city,identity,scoreLevel,"
+						"serviceStatus,carrierStatus,"
+						"relationStatus,carrier,groupid,gender);");
+		if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+			debug_error("create table contacts:%s",
+							sqlite3_errmsg(db));
+			sqlite3_close(db);
+			return;
+		}
+	}
+	foreach_contactlist(user->contactList, pos){
+		snprintf(sql, sizeof(sql)-1, "insert into contacts "
+				"values ('%s','%s','%s','%s','%s','%s',"
+				"'%s','%s','%s','%s','%s','%s','%s','%d',%d,"
+				"%d,%d,%d,'%s',%d,%d);",
+				pos->userId, pos->sId, pos->sipuri,
+				has_special_word(pos->localname) ? "": pos->localname,
+			   	has_special_word(pos->nickname)? "": pos->nickname,
+				has_special_word(pos->impression)?"": pos->impression,
+			   	pos->mobileno,
+				pos->devicetype, pos->portraitCrc,
+				pos->birthday, pos->country, pos->province,
+				pos->city, pos->identity, pos->scoreLevel,
+				pos->serviceStatus, pos->carrierStatus,
+				pos->relationStatus, pos->carrier,
+				pos->groupid, pos->gender);
+		if(sqlite3_exec(db, sql, NULL, NULL, &errMsg))
+			debug_error("insert contact %s:%s\n%s",
+							pos->userId, errMsg ? errMsg : "", sql);
+	}
+	/* begin transaction */
+	if(sqlite3_exec(db, "END TRANSACTION;", 0,0, &errMsg)){
+		debug_error("end transaction :%s", errMsg);
+		sqlite3_close(db);
+		return;
+	}
+	sqlite3_close(db);
+	debug_info("Save contact list successfully");
+}
+
+void fetion_contact_update(User *user, Contact *contact)
+{
+	char path[256];
+	char sql[4096];
+	sqlite3 *db;
+	char *errMsg = NULL;
+	Config *config = user->config;
+
+	debug_info("Update contact information");
+
+	sprintf(path , "%s/data.db" , config->userPath);
+	if(sqlite3_open(path, &db)){
+		debug_error("failed to load user list");
+		return;
+	}
+
+	snprintf(sql, sizeof(sql)-1, "update contacts set "
+			"userId='%s',sId='%s',sipuri='%s',"
+			"localname='%s',nickname='%s',"
+			"impression='%s',mobileno='%s',"
+			"devicetype='%s',portraitCrc='%s',"
+			"birthday='%s',country='%s',"
+			"province='%s',city='%s',"
+			"identity=%d,scoreLevel=%d,"
+			"serviceStatus=%d,carrierStatus=%d,"
+			"relationStatus=%d,carrier='%s',"
+			"groupid=%d,gender=%d where userId='%s'",
+			contact->userId, contact->sId, contact->sipuri,
+			contact->localname, contact->nickname,
+			contact->impression, contact->mobileno,
+			contact->devicetype, contact->portraitCrc,
+			contact->birthday, contact->country, contact->province,
+			contact->city, contact->identity, contact->scoreLevel,
+			contact->serviceStatus, contact->carrierStatus,
+			contact->relationStatus, contact->carrier,
+			contact->groupid, contact->gender, contact->userId);
+
+	if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+		debug_error("update contact %s:%s",
+					contact->userId, errMsg ? errMsg : "");
+		sprintf(sql, "create table contacts (userId,"
+					"sId,sipuri,localname,nickname,"
+					"impression,mobileno,devicetype,"
+					"portraitCrc,birthday,country,"
+					"province,city,identity,scoreLevel,"
+					"serviceStatus,carrierStatus,"
+					"relationStatus,carrier,groupid,gender);");
+		if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+			debug_error("create table contacts:%s",
+						sqlite3_errmsg(db));
+			sqlite3_close(db);
+			return;
+		}
+
+		snprintf(sql, sizeof(sql)-1, "insert into contacts "
+					"values ('%s','%s','%s','%s','%s','%s',"
+					"'%s','%s','%s','%s','%s','%s','%s','%d',%d,"
+					"%d,%d,%d,'%s',%d,%d);",
+					contact->userId, contact->sId, contact->sipuri,
+					contact->localname, contact->nickname,
+					contact->impression, contact->mobileno,
+					contact->devicetype, contact->portraitCrc,
+					contact->birthday, contact->country, contact->province,
+					contact->city, contact->identity, contact->scoreLevel,
+					contact->serviceStatus, contact->carrierStatus,
+					contact->relationStatus, contact->carrier,
+					contact->groupid, contact->gender);
+
+		if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+			debug_error("insert contacts:%s",
+							sqlite3_errmsg(db));
+			sqlite3_close(db);
+			return;
+		}
+	}
+	sqlite3_close(db);
+}
+
+int fetion_contact_del_localbuddy(User *user, const char *userid)
+{
+	char path[256];
+	char sql[4096];
+	sqlite3 *db;
+	char *errMsg = NULL;
+	Config *config = user->config;
+
+	sprintf(path , "%s/data.db" , config->userPath);
+	if(sqlite3_open(path, &db)){
+		debug_error("failed to delete localbuddy");
+		return -1;
+	}
+
+	sprintf(sql, "delete from contacts where "
+			"userid='%s';", userid);
+	if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+		debug_error("failed to delete localbuddy:%s",
+				errMsg);
+		return -1;
+	}
+	return 1;
+}
+
+int fetion_contact_del_localgroup(User *user, const char *userid)
+{
+	char path[256];
+	char sql[4096];
+	sqlite3 *db;
+	char *errMsg = NULL;
+	Config *config = user->config;
+
+	sprintf(path , "%s/data.db" , config->userPath);
+	if(sqlite3_open(path, &db)){
+		debug_error("failed to delete localgroup");
+		return -1;
+	}
+
+	sprintf(sql, "delete from groups where "
+			"id='%s';", userid);
+	if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+		debug_error("failed to delete localgroup:%s",
+				errMsg);
+		return -1;
+	}
+	return 1;
+}
+
+static int has_special_word(const char *in)
+{
+	int i = 0;
+	for(;i< strlen(in); i++){
+		if(in[i] == '\'')
+			return 1;
+	}
+	return 0;
+}
