@@ -20,7 +20,6 @@
 
 #include <openfetion.h>
 
-char buffer[1025];
 int callid = 1;
 
 FetionSip* fetion_sip_new(FetionConnection* tcp , const char* sid)
@@ -200,22 +199,20 @@ void fetion_sip_add_header(FetionSip* sip , SipHeader* header)
 
 char* fetion_sip_to_string(FetionSip* sip , const char* body)
 {
-	char *res , *head , buf[40] , type[5];
+	char *res , *head , buf[1024] , type[128];
 	SipHeader *pos , *tmp;
 	int len = 0;
 
 	pos = sip->header;
-	while(pos != NULL)
-	{
+	while(pos){
 		len += (strlen(pos->value) + strlen(pos->name) + 5);
 		pos = pos->next;
 	}
 	len += (body == NULL ? 100 : strlen(body) + 100 );
 	res = (char*)malloc(len + 1);
 	memset(res , 0 , len + 1);
-	bzero(type , sizeof(type));
-	switch(sip->type)
-	{
+	memset(type, 0 , sizeof(type));
+	switch(sip->type){
 		case SIP_REGISTER     : strcpy(type , "R");
 			break;
 		case SIP_SUBSCRIPTION :	strcpy(type , "SUB");
@@ -235,27 +232,25 @@ char* fetion_sip_to_string(FetionSip* sip , const char* body)
 		default:
 			break;
 	};
+
 	if(strlen(type) == 0)
 		return NULL;
-	bzero(buf , sizeof(buf));
-	sprintf(buf , "%s fetion.com.cn SIP-C/4.0\r\n" , type);
-	strcpy(res , buf);
-	bzero(buf , sizeof(buf));
-	sprintf(buf , "F: %s\r\n" , sip->from);
-	strcat(res , buf);
-	bzero(buf , sizeof(buf));
-	sprintf(buf , "I: %d\r\n" , sip->callid);
-	strcat(res , buf);
-	bzero(buf , sizeof(buf));
-	sprintf(buf , "Q: 2 %s\r\n" , type);
+
+	sprintf(buf, "%s fetion.com.cn SIP-C/4.0\r\n"
+			"F: %s\r\n"
+			"I: %d\r\n"
+			"Q: 2 %s\r\n",
+			type,
+			sip->from,
+			sip->callid,
+			type);
+
 	strcat(res , buf);
 
 	pos = sip->header;
-	while(pos != NULL)
-	{
+	while(pos){
 		len = strlen(pos->value) + strlen(pos->name) + 5;
 		head = (char*)malloc(len);
-		memset(head , 0 , len);
 		sprintf(head , "%s: %s\r\n" , pos->name , pos->value);
 		strcat(res , head);
 		tmp = pos;
@@ -264,15 +259,11 @@ char* fetion_sip_to_string(FetionSip* sip , const char* body)
 		free(tmp->value);
 		free(tmp);
 	}
-	if(body != NULL)
-	{
-		bzero(buf , sizeof(buf));
+	if(body){
 		sprintf(buf , "L: %d\r\n\r\n" , strlen(body));
 		strcat(res , buf);
 		strcat(res , body);
-	}
-	else
-	{
+	}else{
 		strcat(res , "\r\n");
 	}
 	callid ++;
@@ -315,10 +306,10 @@ char* fetion_sip_get_pgid_by_sipuri(const char *pgsipuri)
 
 int fetion_sip_get_attr(const char* sip , const char* name , char* result)
 {
-	char m_name[5];
+	char m_name[16];
 	char* pos;
 	int n;
-	bzero(m_name , sizeof(m_name));
+
 	sprintf(m_name , "%s: " , name);
 	if(strstr(sip , m_name) == NULL)
 		return -1;
@@ -344,6 +335,8 @@ int fetion_sip_get_code(const char* sip)
 {
 	char *pos , res[32];
 	int n;
+
+	memset(res, 0, sizeof(res));
 	if(strstr(sip , "4.0 ") == NULL)
 	    return 400;
 	pos = strstr(sip , "4.0 ") + 4;
@@ -357,6 +350,10 @@ int fetion_sip_get_type(const char* sip)
 {
 	char res[128];
 	int n;
+
+	if(!strstr(sip, " "))
+		return SIP_UNKNOWN;
+
 	n = strlen(sip) - strlen(strstr(sip , " "));
 	bzero(res , sizeof(res));
 	strncpy(res , sip , n);
@@ -380,7 +377,7 @@ char* fetion_sip_get_response(FetionSip* sip)
 {
 	char *res;
 	unsigned int len , n , c , c1;
-	char buf[2048];
+	char buf[4096];
 
 	memset(buf , 0 , sizeof(buf));
 
@@ -391,24 +388,30 @@ char* fetion_sip_get_response(FetionSip* sip)
 
 	len = fetion_sip_get_length(buf);
 
-	if(strstr(buf , "\r\n\r\n") == NULL)
-		return NULL;
+	while(strstr(buf , "\r\n\r\n") == NULL && c < sizeof(buf))
+		c += tcp_connection_recv(sip->tcp, buf + c, sizeof(buf) - c - 1);
+	
+
 	n = strlen(buf) - strlen(strstr(buf , "\r\n\r\n") + 4);
 	len += n;
 	res = (char*)malloc(len + 1);
-	memset(res , 0 , len + 1);
 	if(res == NULL)
 		return NULL;
+	memset(res , 0 , len + 1);
 	strcpy(res , buf);
 	if(c < len)
-	    	for(;;){
+	   	for(;;){
 			memset(buf , 0 , sizeof(buf));
 			c1 = tcp_connection_recv(sip->tcp , buf
 					, len -c < (sizeof(buf) - 1) ? len -c : (sizeof(buf) - 1) );
+			if(c1 == -1){
+				free(res);
+				return NULL;
+			}
 
-			strcpy(res + c, buf);
+			strncpy(res + c, buf, c1);
 			c += c1;
-			if(c == len)
+			if(c >= len)
 				break;
 		}
 	return res;
@@ -417,10 +420,134 @@ void fetion_sip_set_connection(FetionSip* sip , FetionConnection* conn)
 {
 	sip->tcp = conn;
 }
-SipMsg* fetion_sip_listen(FetionSip* sip)
+
+static SipMsg *sipmsg_new(void)
+{
+	SipMsg *msg = (SipMsg*)malloc(sizeof(SipMsg));
+	memset(msg, 0, sizeof(SipMsg));
+	msg->next = NULL;
+	return msg;
+}
+
+static void sipmsg_set_msg(SipMsg *sipmsg, const char *msg, int n)
+{
+	sipmsg->message = (char*)malloc(n + 1);
+	memset(sipmsg->message, 0, n + 1);
+	strncpy(sipmsg->message, msg, n);
+}
+
+#define BUFFER_SIZE (1024 * 20)
+
+/* the following code is hard to read,forgive me! */
+SipMsg *fetion_sip_listen(FetionSip *sip, int *error)
+{
+	int     n;
+	int     body_len;
+	char    buffer[BUFFER_SIZE];
+	char    holder[BUFFER_SIZE];
+	char   *cur;
+	char   *pos;
+	SipMsg *list = NULL;
+	SipMsg *msg;
+
+	*error = 0;
+
+	memset(buffer, 0, sizeof(buffer));
+	n = tcp_connection_recv_dont_wait(sip->tcp,
+				buffer, sizeof(buffer) - 1);
+	if(n == 0){
+		debug_info("fetion_sip_listen 0");
+		*error = 1;
+		return NULL;
+	}
+	if(n == -1){
+		*error = 1;
+		return NULL;
+	}
+	cur = buffer;
+	for(;;){
+		pos = strstr(cur, "\r\n\r\n");
+		body_len = 0;
+		if(pos){
+			n = strlen(cur) - strlen(pos);
+			memset(holder, 0, sizeof(holder));
+			strncpy(holder, cur, n);
+			body_len = fetion_sip_get_length(holder);
+		}
+
+		if(cur == NULL || strlen(cur) == 0)
+			return list;
+
+		if(body_len == 0 && pos){
+			msg = sipmsg_new();
+			n = strlen(cur) - strlen(pos) + 4;
+			sipmsg_set_msg(msg, cur, n);
+			if(list)
+				fetion_sip_message_append(list, msg);
+			else
+				list = msg;
+			cur += n;
+			continue;
+		}
+
+		if((body_len == 0 && !pos) ||
+				(body_len != 0 && !pos)){
+			memset(holder, 0 , sizeof(holder));
+			strcpy(holder, cur);
+			tcp_connection_recv(sip->tcp,
+					holder + strlen(cur),
+					BUFFER_SIZE - strlen(cur) - 1);
+			memset(buffer, 0 , sizeof(buffer));
+			strcpy(buffer, holder);
+			cur = buffer;
+			continue;
+		}else{
+			/* now body_len != 0 */
+			pos += 4;
+			memset(holder, 0 , sizeof(holder));
+			if(strlen(pos) < body_len){
+				strcpy(holder, cur);
+				tcp_connection_recv(sip->tcp,
+					holder + strlen(cur),
+					BUFFER_SIZE - strlen(cur) - 1);
+				memset(buffer, 0, sizeof(buffer));
+				strcpy(buffer, holder);
+				cur = buffer;
+				continue;
+			}else if(strlen(pos) == body_len){
+				msg = sipmsg_new();
+				sipmsg_set_msg(msg, cur, strlen(cur));
+				if(list)
+					fetion_sip_message_append(list, msg);
+				else
+					list = msg;
+				return list;
+			}else{
+				msg = sipmsg_new();
+				n = strlen(cur) - strlen(pos) + body_len;
+				sipmsg_set_msg(msg, cur, n);
+				if(list)
+					fetion_sip_message_append(list, msg);
+				else
+					list = msg;
+
+				memset(holder, 0 , sizeof(holder));
+				strcpy(holder, cur + n);
+				memset(buffer, 0, sizeof(buffer));
+				strcpy(buffer, holder);
+				cur = buffer;
+				continue;
+			}
+		}
+	}
+
+}
+
+SipMsg* fetion_sip_listen1(FetionSip* sip)
 {
 	char *buf , *pos , *missingStr = NULL , *tmp = NULL;
 	char *strBeforeSpt = NULL;
+	char buffer[1025];
 	unsigned int strBeforeSptLen = 0;
 	unsigned int len , msglen , missingLen , rereceiveLen , msgCurrentLen;
 	int bodyLen;
@@ -438,10 +565,8 @@ SipMsg* fetion_sip_listen(FetionSip* sip)
 		msg->next = NULL;
 		bodyLen = fetion_sip_get_length(buf);
 		pos = strstr(buf , "\r\n\r\n");
-		/**
-		 * ensure the bodyLength is read from 
-		 * the first chunk before '\r\n\r\n'
-		 */
+		/* ensure the bodyLength is read from 
+		 * the first chunk before '\r\n\r\n' */
 		if(pos != NULL)
 		{
 			strBeforeSptLen = strlen(buf) - strlen(pos) + 1;
@@ -466,10 +591,10 @@ SipMsg* fetion_sip_listen(FetionSip* sip)
 		 */
 		if(bodyLen == 0 && pos != NULL)
 		{
-			len = strlen(buf) - strlen(pos);
-			msg->message = (char*)malloc(len + 5);
-			bzero(msg->message , len + 5);
-			strncpy(msg->message , buf , len + 4);
+			len = strlen(buf) - strlen(pos) + 4;
+			msg->message = (char*)malloc(len + 1);
+			memset(msg->message, 0 , len + 1);
+			strncpy(msg->message , buf , len);
 			if(msglist == NULL)
 				msglist = msg;
 			else
@@ -487,8 +612,7 @@ SipMsg* fetion_sip_listen(FetionSip* sip)
 		 */
 		if(bodyLen == 0 && pos == NULL)
 		{
-			if(tmp != NULL && strlen(tmp) != 0)
-			{
+			if(tmp != NULL){
 				free(tmp);
 				tmp = NULL;
 			}
@@ -496,7 +620,7 @@ SipMsg* fetion_sip_listen(FetionSip* sip)
 			memset(tmp , 0 , strlen(buf) + sizeof(buffer));
 			strcpy(tmp , buf);
 			memset(buffer , 0 , sizeof(buffer));
-			tcp_connection_recv(sip->tcp , buffer , 1024);
+			tcp_connection_recv(sip->tcp , buffer , sizeof(buffer) - 1);
 			strcat(tmp , buffer);
 			buf = tmp;
 			continue;
@@ -516,7 +640,7 @@ SipMsg* fetion_sip_listen(FetionSip* sip)
 			memset(tmp , 0 , strlen(buf) + sizeof(buffer));
 			strcpy(tmp , buf);
 			memset(buffer , 0 , sizeof(buffer));
-			tcp_connection_recv(sip->tcp , buffer , 1024);
+			tcp_connection_recv(sip->tcp , buffer , sizeof(buffer) - 1);
 			strcat(tmp , buffer);
 			buf = tmp;
 			continue;
@@ -566,11 +690,13 @@ SipMsg* fetion_sip_listen(FetionSip* sip)
 					msglist = msg;
 				else
 					fetion_sip_message_append(msglist , msg);
+#if 0
 				if(tmp != NULL && strlen(tmp) != 0)
 				{
 					free(tmp);
 					tmp = NULL;
 				}
+#endif
 				return msglist;
 			}
 			else
@@ -679,7 +805,7 @@ void fetion_sip_parse_message(FetionSip* sip , const char* sipmsg , Message** ms
 {
 	char len[16] , callid[16] , sequence[16] ;
 	char sendtime[32] , from[64] , rep[256];
-	char memsipuri[64] , sysback[256];
+	char memsipuri[64];
 
 	char *pos = NULL;
 	xmlDocPtr doc;
@@ -694,12 +820,13 @@ void fetion_sip_parse_message(FetionSip* sip , const char* sipmsg , Message** ms
 	fetion_sip_get_attr(sipmsg , "I" , callid);
 	fetion_sip_get_attr(sipmsg , "Q" , sequence);
 	fetion_sip_get_attr(sipmsg , "D" , sendtime);	
-	fetion_sip_get_attr(sipmsg , "BK" , sysback);
 
 	*msg = fetion_message_new();
 
-	if(strlen(sysback))
-		(*msg)->sysback = atoi(sysback);
+	(*msg)->sysback = 0;
+	if(strstr(sipmsg, "SIP-C/3.0") &&
+		!strstr(sipmsg, "SIP-C/4.0"))
+		(*msg)->sysback = 1;
 
 	/* group message */
 	if(strstr(from , "PG") != NULL){
@@ -725,7 +852,7 @@ void fetion_sip_parse_message(FetionSip* sip , const char* sipmsg , Message** ms
 		fetion_message_set_message(*msg , pos);
 	}
 
-	bzero(rep , sizeof(rep));
+	memset(rep, 0, sizeof(rep));
 	if(strstr(from , "PG") == NULL)
 	    sprintf(rep ,"SIP-C/4.0 200 OK\r\nF: %s\r\nI: %s\r\nQ: %s\r\n\r\n"
 				    , from , callid , sequence );
@@ -929,8 +1056,6 @@ static char *generate_action_accept_body(Share *share)
 	xmlNodePtr node , root;
 	char body[] = "<share-content></share-content>";
 
-	DEBUG_FOOTPRINT();
-
 	doc = xmlParseMemory(body , strlen(body));
 	root = xmlDocGetRootElement(doc);
 	node = xmlNewChild(root , NULL , BAD_CAST "client" , NULL);
@@ -1089,10 +1214,10 @@ void fetion_sip_get_auth_attr(const char* auth , char** ipaddress , int* port ,
 	char* pos = strstr(auth , "address=\"") + 9;
 	int n = strlen(pos) - strlen(strstr(pos , ":"));
 	char port_str[6] = { 0 };
-	*credential = (char*)malloc(48);
-	memset(*credential , 0 , 48);
-	*ipaddress = (char*)malloc(18);
-	memset(*ipaddress , 0 , 18);
+	*credential = (char*)malloc(256);
+	memset(*credential , 0 , 256);
+	*ipaddress = (char*)malloc(256);
+	memset(*ipaddress , 0 , 256);
 	strncpy(*ipaddress , pos , n);
 	pos = strstr(pos , ":") + 1;
 	n = strlen(pos) - strlen(strstr(pos , ";"));
