@@ -27,8 +27,6 @@ FxMyself* fx_myself_new(FxMain* fxmain)
 {
 	FxMyself* fxmyself = (FxMyself*)malloc(sizeof(FxMyself));
 
-	DEBUG_FOOTPRINT();
-
 	memset(fxmyself , 0 , sizeof(FxMyself));
 	fxmyself->fxmain = fxmain;
 	fxmyself->conv = fetion_conversation_new(fxmain->user , NULL , NULL);
@@ -43,9 +41,6 @@ void fx_myself_bind(FxMyself* fxmyself)
 	GdkPixbuf* pb;
 	char name[128];
 
-	DEBUG_FOOTPRINT();
-
-	bzero(name , sizeof(name));
 	sprintf(name , "%s/%s.jpg" , config->iconPath ,
 			user->sId );
 	pb = gdk_pixbuf_new_from_file_at_size(name , 40 , 40 ,
@@ -71,15 +66,13 @@ void fx_myself_add_message(FxMyself* fxmyself , const char* message)
 	GtkTextIter iter;
 
 	User* user = fxmyself->fxmain->user;
-	char text[200] = { 0 };
+	char text[4096];
 	char time[30] = { 0 };
 	char color[] = "blue";
 
 	struct tm* sendtime = get_currenttime();
 	GtkTextBuffer* buffer =	gtk_text_view_get_buffer(GTK_TEXT_VIEW(fxmyself->recv_text));
 
-	DEBUG_FOOTPRINT();
-
 	strftime(time , sizeof(time) , "%H:%M:%S" , sendtime);
 
 	sprintf(text , "%s(%s) %s\n" , user->nickname , user->sId , time);
@@ -110,8 +103,6 @@ void fx_myself_initialize(FxMyself* fxmyself)
 	GtkWidget* vbox;
 	GtkWidget* action_area;
 
-	DEBUG_FOOTPRINT();
-
 	fxmyself->dialog = gtk_dialog_new(); 
 	vbox = GTK_DIALOG(fxmyself->dialog)->vbox;
 	action_area = GTK_DIALOG(fxmyself->dialog)->action_area;
@@ -159,6 +150,7 @@ void fx_myself_initialize(FxMyself* fxmyself)
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(fxmyself->recv_scroll)
 									  , GTK_SHADOW_ETCHED_IN);
 	fxmyself->recv_text = gtk_text_view_new();
+	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(fxmyself->recv_text) , FALSE);
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(fxmyself->recv_text) , GTK_WRAP_CHAR);
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(fxmyself->recv_text) , FALSE);
 	gtk_container_add(GTK_CONTAINER(fxmyself->recv_scroll) , fxmyself->recv_text);
@@ -199,8 +191,12 @@ void fx_myself_initialize(FxMyself* fxmyself)
 	g_signal_connect(send_button , "clicked" , G_CALLBACK(fx_myself_on_send_clicked) , fxmyself);
 
 	gtk_window_set_position(GTK_WINDOW(fxmyself->dialog) , GTK_WIN_POS_CENTER);
-	gtk_window_set_opacity(GTK_WINDOW(fxmyself->dialog) , 0.9);
+	//gtk_window_set_opacity(GTK_WINDOW(fxmyself->dialog) , 0.9);
 	fx_myself_bind(fxmyself);
+
+	GTK_WIDGET_SET_FLAGS(fxmyself->send_text, GTK_CAN_FOCUS);
+	gtk_widget_grab_focus(fxmyself->send_text);
+
 	gtk_widget_show_all(fxmyself->dialog);
 	gtk_widget_hide(fxmyself->dialog);
 }
@@ -208,8 +204,6 @@ void fx_myself_on_close_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 {
 	FxMyself* fxmyself = (FxMyself*)data;
 
-	DEBUG_FOOTPRINT();
-
 	gtk_dialog_response(GTK_DIALOG(fxmyself->dialog) , GTK_RESPONSE_OK);
 }
 
@@ -220,8 +214,6 @@ void fx_myself_on_send_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 	GtkTextIter begin , end;
 	char* text;
 
-	DEBUG_FOOTPRINT();
-
 	gtk_text_buffer_get_start_iter(fxmyself->send_buffer , &begin);
 	gtk_text_buffer_get_end_iter(fxmyself->send_buffer , &end);
 	text = gtk_text_buffer_get_text(fxmyself->send_buffer , &begin , &end , TRUE);
@@ -237,8 +229,6 @@ gboolean fx_myself_on_enter_pressed(GtkWidget* widget , GdkEventKey* event , gpo
 	FxMyself* fxmyself = NULL;
 	Config *config = NULL;
 	
-	DEBUG_FOOTPRINT();
-
 	if(event->keyval == GDK_Return || event->keyval == GDK_ISO_Enter || event->keyval == GDK_KP_Enter)
 	{
 		fxmyself = (FxMyself*)data;
