@@ -22,27 +22,22 @@
 
 static void fx_set_on_ok_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 {
-	FxSet *fxset = (FxSet*)data;
-	User *user = fxset->fxmain->user;
-	Config *config = user->config;
-	/**
-	 * system setting varibles
-	 */
-	GtkTextView *textview = GTK_TEXT_VIEW(fxset->apEty);
+	FxSet         *fxset = (FxSet*)data;
+	User          *user = fxset->fxmain->user;
+	Config        *config = user->config;
+	/* system setting varibles */
+	GtkTextView   *textview = GTK_TEXT_VIEW(fxset->apEty);
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer(textview);
-	GtkTextIter startIter , endIter;
-	const char *autoReplyMsg = NULL;
-	/**
-	 * personal setting varibles
-	 */
-	const char *nickname = NULL;
-	const char *impression = NULL;
-	char nickname_text[1024];
-	int gender;
-	GtkTreeModel *genderModel = NULL;
-	GtkTreeIter genderIter;
-
-	DEBUG_FOOTPRINT();
+	GtkTextIter    startIter;
+   	GtkTextIter    endIter;
+	const gchar   *autoReplyMsg = NULL;
+	/* personal setting varibles */
+	const gchar   *nickname = NULL;
+	const gchar   *impression = NULL;
+	gchar          nickname_text[1024];
+	gint           gender;
+	GtkTreeModel  *genderModel = NULL;
+	GtkTreeIter    genderIter;
 
 	if(gtk_notebook_get_current_page(GTK_NOTEBOOK(fxset->notebook)) == PAGE_SYSTEM)
 	{
@@ -71,11 +66,22 @@ static void fx_set_on_ok_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 		else
 			config->msgAlert = MSG_ALERT_ENABLE;
 
-		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fxset->iconBtn)))
+		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fxset->iconBtn))){
 			config->canIconify = ICON_CAN;
-		else
+			config->closeAlert = CLOSE_ALERT_DISABLE;
+		}else
 			config->canIconify = ICON_CANNOT;
 
+		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fxset->autoAwayBtn)))
+			config->autoAway = AUTO_AWAY_ENABLE;
+		else
+			config->autoAway = AUTO_AWAY_DISABLE;
+
+		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fxset->onlineNotifyBtn)))
+			config->onlineNotify = ONLINE_NOTIFY_ENABLE;
+		else
+			config->onlineNotify = ONLINE_NOTIFY_DISABLE;
+
 		gtk_text_buffer_get_start_iter(buffer , &startIter);
 		gtk_text_buffer_get_end_iter(buffer , &endIter);
 		autoReplyMsg = gtk_text_buffer_get_text(buffer , &startIter , &endIter , TRUE);
@@ -108,8 +114,6 @@ static void fx_set_on_ok_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 		if(fetion_user_update_info(user) > 0)
 		{
 			
-			bzero(nickname_text , sizeof(nickname_text));
-
 			sprintf(nickname_text , "<b>%s</b>"
 					, user->nickname == NULL ? user->sId : user->nickname );
 
@@ -129,8 +133,6 @@ static void fx_set_on_ok_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 
 static void fx_set_on_cancel_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 {
-	DEBUG_FOOTPRINT();
-
 	gtk_dialog_response(GTK_DIALOG(data) , GTK_RESPONSE_CANCEL);
 }
 
@@ -138,24 +140,16 @@ static void fx_set_on_autoreply_toggled(GtkWidget *widget , gpointer data)
 {
 	FxSet *fxset = (FxSet*)data;
 
-	DEBUG_FOOTPRINT();
-
 	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-	{
 		gtk_widget_set_sensitive(fxset->apEty , TRUE);
-	}
 	else
-	{
 		gtk_widget_set_sensitive(fxset->apEty , FALSE);
-	}
 }
 
 FxSet* fx_set_new(FxMain* fxmain)
 {
 	FxSet *fxset = (FxSet*)malloc(sizeof(FxSet));
 
-	DEBUG_FOOTPRINT();
-
 	memset(fxset , 0 , sizeof(FxSet));
 	fxset->fxmain = fxmain;
 	return fxset;
@@ -172,8 +166,6 @@ void fx_set_initialize(FxSet* fxset)
 	GdkPixbuf* pb = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"online.svg",
 				   22, 22, NULL);
 
-	DEBUG_FOOTPRINT();
-
 	fxset->dialog = gtk_dialog_new();
 	gtk_window_set_icon(GTK_WINDOW(fxset->dialog) , pb);
 	g_object_unref(pb);
@@ -225,8 +217,6 @@ void fx_set_bind_system(FxSet* fxset)
 	GtkTextIter startIter , endIter;
 	char *autoReplyMsg = NULL;
 
-	DEBUG_FOOTPRINT();
-
 	if(config->sendMode == SEND_MODE_ENTER)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->etBtn) , TRUE);	
 	else
@@ -257,6 +247,16 @@ void fx_set_bind_system(FxSet* fxset)
 	else
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->iconBtn) , FALSE);
 
+	if(config->autoAway == AUTO_AWAY_ENABLE)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->autoAwayBtn) , TRUE);
+	else
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->autoAwayBtn) , FALSE);
+
+	if(config->onlineNotify == ONLINE_NOTIFY_ENABLE)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->onlineNotifyBtn), TRUE);
+	else
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->onlineNotifyBtn), FALSE);
+
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(fxset->apEty));
 	gtk_text_buffer_get_start_iter(buffer , &startIter);
 	gtk_text_buffer_get_end_iter(buffer , &endIter);
@@ -289,8 +289,6 @@ void fx_set_initialize_personal(FxSet* fxset)
 	char *cityName = NULL;
 	char *provinceName = NULL;
 
-	DEBUG_FOOTPRINT();
-
 	box = gtk_fixed_new();
 	gtk_box_pack_start_defaults(GTK_BOX(fxset->psetting) , box);
 
@@ -380,24 +378,26 @@ void fx_set_initialize_personal(FxSet* fxset)
 
 
 	gtk_entry_set_text(GTK_ENTRY(fxset->sid_entry) , user->sId);
-	if(user->mobileno != NULL)
+	if(user->mobileno)
 		gtk_entry_set_text(GTK_ENTRY(fxset->mno_entry) , user->mobileno);
-	if(user->nickname != NULL)
+
+	if(user->nickname)
 		gtk_entry_set_text(GTK_ENTRY(fxset->nick_entry) , user->nickname);
-	if(user->impression != NULL)
+
+	if(user->impression)
 		gtk_entry_set_text(GTK_ENTRY(fxset->impre_entry) , user->impression);
-	if(user->province != NULL)
-	{
-		provinceName = fetion_config_get_province_name(user->province);
+
+	provinceName = fetion_config_get_province_name(user->province);
+
+	if(provinceName)
 		gtk_entry_set_text(GTK_ENTRY(fxset->province_entry) , provinceName);
-		free(provinceName);
-	}
-	if(user->city != NULL)
-	{
-		cityName = fetion_config_get_city_name(user->province , user->city);
+	free(provinceName);
+
+	cityName = fetion_config_get_city_name(user->province , user->city);
+
+	if(cityName)
 		gtk_entry_set_text(GTK_ENTRY(fxset->city_entry) , cityName);
-		free(cityName);
-	}
+	free(cityName);
 
 	gtk_entry_set_text(GTK_ENTRY(fxset->job_entry) , _("Secrecy"));
 	gtk_entry_set_text(GTK_ENTRY(fxset->email_entry) , _("Secrecy"));
@@ -432,8 +432,6 @@ void fx_set_initialize_system(FxSet* fxset)
 	GtkWidget *apScr = NULL;
 	GSList	  *gl = NULL;
 	
-	DEBUG_FOOTPRINT();
-
 	fixed = gtk_fixed_new();
 
 	label1 = gtk_label_new("");
@@ -455,17 +453,23 @@ void fx_set_initialize_system(FxSet* fxset)
 	fxset->muteBtn = gtk_check_button_new_with_label(_("Mute"));
 	gtk_fixed_put(GTK_FIXED(fixed) , fxset->muteBtn , 40 , 92);
 
+	fxset->autoAwayBtn = gtk_check_button_new_with_label(_("Auto away when idle"));
+	gtk_fixed_put(GTK_FIXED(fixed) , fxset->autoAwayBtn , 230 , 92);
+
+	fxset->onlineNotifyBtn = gtk_check_button_new_with_label(_("Online notification enabled"));
+	gtk_fixed_put(GTK_FIXED(fixed) , fxset->onlineNotifyBtn , 40 , 117);
+
 	label2 = gtk_label_new("");
 	gtk_label_set_markup(GTK_LABEL(label2) , _("<b>Auto Reply</b>"));
-	gtk_fixed_put(GTK_FIXED(fixed) , label2 , 20 , 127);
+	gtk_fixed_put(GTK_FIXED(fixed) , label2 , 20 , 142);
 
 	fxset->apBtn = gtk_check_button_new_with_label(_("Enable"));
 	g_signal_connect(fxset->apBtn , "toggled" , G_CALLBACK(fx_set_on_autoreply_toggled) , fxset);
-	gtk_fixed_put(GTK_FIXED(fixed) , fxset->apBtn , 140 , 124);
+	gtk_fixed_put(GTK_FIXED(fixed) , fxset->apBtn , 140 , 142);
 
 	fxset->apEty = gtk_text_view_new();
 	gtk_widget_set_sensitive(fxset->apEty , FALSE);
-	gtk_widget_set_usize(fxset->apEty , 380 , 50);
+	gtk_widget_set_usize(fxset->apEty , 380 , 40);
 	apScr = gtk_scrolled_window_new(NULL , NULL);
 	gtk_container_add(GTK_CONTAINER(apScr) , fxset->apEty);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(apScr)
@@ -473,7 +477,7 @@ void fx_set_initialize_system(FxSet* fxset)
 								 , GTK_POLICY_NEVER);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(apScr)
 									  , GTK_SHADOW_ETCHED_IN);
-	gtk_fixed_put(GTK_FIXED(fixed) , apScr , 40 , 150);
+	gtk_fixed_put(GTK_FIXED(fixed) , apScr , 40 , 165);
 
 	label3 = gtk_label_new("");
 	gtk_label_set_markup(GTK_LABEL(label3) , _("<b>Send Message</b>"));
@@ -509,8 +513,6 @@ GtkTreeModel* fx_set_create_gender_model()
 	GtkTreeIter iter;
 	int i;
 
-	DEBUG_FOOTPRINT();
-
 	for(i = 0; genders[i].name != NULL ; i++)
 	{
 		gtk_tree_store_append(model , &iter , NULL);
